import * as React from 'react';
import React__default from 'react';
import { ClientSettings, ReplicaCache, parseAuthorAddress, isErr, parseShareAddress } from 'earthstar';
import { useSyncExternalStoreWithSelector } from 'use-sync-external-store/shim/with-selector';
import { identicon } from 'minidenticons';

const ClientSettingsContext = React.createContext(new ClientSettings());
/** Subscribe to a given peer's replicas as they change. */
function usePeerReplicas(peer) {
    const [replicas, setReplicas] = React.useState(peer.replicas());
    React.useEffect(() => {
        const unsub = peer.onReplicasChange((replicas) => {
            setReplicas(Array.from(replicas.values()));
        });
        return () => {
            unsub();
        };
    }, [peer]);
    return replicas;
}
/** Subscribe to an `Earthstar.ClientSetting`'s author.
 *
 * @returns A tuple where the first element is the author, and the second element a function to update the author.
 */
function useAuthorSettings() {
    const settings = React.useContext(ClientSettingsContext);
    const [author, setAuthor] = React.useState(() => settings.author);
    React.useEffect(() => {
        const unsub = settings.onAuthorChanged((newAuthor) => {
            setAuthor(newAuthor);
        });
        return unsub;
    }, [settings]);
    const set = React.useCallback((newAuthor) => {
        settings.author = newAuthor;
    }, [settings]);
    return [author, set];
}
function useShareSettings() {
    const settings = React.useContext(ClientSettingsContext);
    const [shares, setShares] = React.useState(() => settings.shares);
    React.useEffect(() => {
        const unsub = settings.onSharesChanged((newShares) => {
            setShares(newShares);
        });
        return unsub;
    }, [settings]);
    const addShare = React.useCallback((addr) => {
        return settings.addShare(addr);
    }, [settings]);
    const removeShare = React.useCallback((addr) => {
        return settings.removeShare(addr);
    }, [settings]);
    return [shares, addShare, removeShare];
}
function useShareSecretSettings() {
    const settings = React.useContext(ClientSettingsContext);
    const [secrets, setSecrets] = React.useState(() => settings.shareSecrets);
    React.useEffect(() => {
        const unsub = settings.onShareSecretsChanged((newSecrets) => {
            setSecrets(newSecrets);
        });
        return unsub;
    }, [settings]);
    const addSecret = React.useCallback(async (addr, secret) => {
        return settings.addSecret(addr, secret);
    }, [settings]);
    const removeSecret = React.useCallback((addr) => {
        return settings.removeSecret(addr);
    }, [settings]);
    return [secrets, addSecret, removeSecret];
}
function useServerSettings() {
    const settings = React.useContext(ClientSettingsContext);
    const [servers, setServers] = React.useState(() => settings.servers);
    React.useEffect(() => {
        const unsub = settings.onServersChanged((newServers) => {
            setServers(newServers);
        });
        return unsub;
    }, [settings]);
    const addServer = React.useCallback((url) => {
        return settings.addServer(url);
    }, [settings]);
    const removeServer = React.useCallback((url) => {
        return settings.removeServer(url);
    }, [settings]);
    return [servers, addServer, removeServer];
}
function useReplica(replica) {
    const cache = React.useMemo(() => {
        return new ReplicaCache(replica, 1000);
    }, [replica]);
    const [version, setVersion] = React.useState(cache.version);
    React.useEffect(() => {
        setVersion(cache.version);
        return cache.onCacheUpdated(() => {
            setVersion(cache.version);
        });
    }, [cache]);
    const snapshot = React.useMemo(() => {
        return { cache, version };
    }, [version, cache]);
    const subscribe = (cb) => {
        return cache.onCacheUpdated(cb);
    };
    const obj = useSyncExternalStoreWithSelector(subscribe, () => snapshot, () => snapshot, (obj) => {
        return obj;
    });
    return obj.cache;
}
/*
export function useSync(
  target: string | Peer,
): [Syncer<undefined, unknown>, SyncerStatus] {
  const peer = usePeer();

  const syncer = peer.sync(target);

  const [status, setStatus] = React.useState(syncer.getStatus);

  React.useEffect(() => {
    return syncer.onStatusChange((status) => {
      setStatus(status);
    });
  }, [syncer]);

  return [syncer, status];
}
*/
/*
export function useInvitation(invitationCode: string) {
  const addShare = React.useContext(AddShareContext);
  const [, setPubs] = useReplicaServers();

  try {
    const url = new URL(invitationCode);

    const isEarthstarURL = url.protocol === "earthstar:";

    if (!isEarthstarURL) {
      return new EarthstarError("Invitation not a valid Earthstar URL");
    }

    const version = url.searchParams.get("v");

    if (version !== "1") {
      return new EarthstarError(
        "Unrecognised Earthstar invitation format version",
      );
    }

    const workspace = url.searchParams.get("workspace");

    if (workspace === null) {
      return new EarthstarError(
        "No workspace found in Earthstar invitation URL",
      );
    }

    const plussedWorkspace = workspace.replace(" ", "+");

    const shareIsValid = checkShareIsValid(plussedWorkspace);

    if (isErr(shareIsValid)) {
      return shareIsValid;
    }

    const pubs = url.searchParams.getAll("pub");

    try {
      pubs.forEach((pubUrl) => new URL(pubUrl));
    } catch {
      return new EarthstarError("Malformed Pub URL found");
    }

    const redeem = (excludedPubs: string[] = []) => {
      addShare(plussedWorkspace);

      const nextPubs = Array.from(
        new Set([
          ...pubs.filter((pubUrl) => !excludedPubs.includes(pubUrl)),
        ]),
      );

      setPubs((prevPubs) =>
        Array.from(
          new Set([
            ...prevPubs,
            ...nextPubs,
          ]),
        )
      );
    };

    return { redeem, workspace: plussedWorkspace, pubs };
  } catch {
    return new EarthstarError("Not a valid Earthstar URL");
  }
}


export function useMakeInvitation(
  shareAddress: string,
  includedPubs: string[] = [],
) {
  const [pubs] = useReplicaServers();
  const address = shareAddress;

  const pubsToUse = pubs.filter((pubUrl) => includedPubs.includes(pubUrl));
  const pubsString = pubsToUse.map((pubUrl) => `&pub=${pubUrl}`).join("");

  if (!address) {
    return "Couldn't create invitation code!";
  }

  return `earthstar:///?workspace=${address}${pubsString}&v=1`;
}
*/

function getAuthorShortName(address) {
    const parsedAuthor = parseAuthorAddress(address);
    if (isErr(parsedAuthor)) {
        return address;
    }
    return parsedAuthor.name;
}
function getShareName(address) {
    const parsedShareAddress = parseShareAddress(address);
    if (isErr(parsedShareAddress)) {
        return address;
    }
    return parsedShareAddress.name;
}

function Identicon({ address, salt, size }) {
    const svg = React__default.useMemo(() => {
        return identicon(`${address}${salt || ""}`);
    }, [address]);
    const withoutMargin = React__default.useMemo(() => {
        return svg.replace('viewBox="-1.5 -1.5 8 8"', 'viewBox="0 0 5 5"');
    }, [svg]);
    return (React__default.createElement("img", { src: `data:image/svg+xml;utf8,${withoutMargin}`, width: size, height: size, alt: `An identicon representing the address ${address}` }));
}

function AuthorLabel({ address, viewingAuthorSecret, iconSize = 10, ...props }) {
    return (React.createElement("span", { ...props, title: address },
        `@${getAuthorShortName(address)}`,
        "\u00A0",
        React.createElement(Identicon, { address: address, size: iconSize, salt: viewingAuthorSecret })));
}

/*

WorkspaceLabel

Given a workspace address, returns an element which:
- Displays the workspace name without the hash (so as not to accidentally expose the address to others)
- Ensures the workspace name is displayed with a preceding + sigil (so as to signify it being a workspace)
- Adds a title attribute with the full workspace address (so the user can check the full address if needed)

*/
function ShareLabel({ address, viewingAuthorSecret, iconSize = 10, ...props }) {
    return (React.createElement("span", { ...props, title: address },
        `+${getShareName(address)}`,
        "\u00A0",
        React.createElement(Identicon, { address: address, size: iconSize, salt: viewingAuthorSecret })));
}

export { AuthorLabel, ClientSettingsContext, ShareLabel, useAuthorSettings, usePeerReplicas, useReplica, useServerSettings, useShareSecretSettings, useShareSettings };
